@@ -1,10 +1,10 @@
-from alembic.ddl.impl import DefaultImpl
-from alembic.ddl.base import alter_table, AddColumn, ColumnName, \
-    format_table_name, format_column_name, ColumnNullable, \
-    format_server_default,ColumnDefault, format_type, ColumnType
-from alembic import util
 from sqlalchemy.ext.compiler import compiles
 
+from .impl import DefaultImpl
+from .base import alter_table, AddColumn, ColumnName, \
+    format_column_name, ColumnNullable, \
+    format_server_default,ColumnDefault, format_type, ColumnType
+
 class OracleImpl(DefaultImpl):
     __dialect__ = 'oracle'
     transactional_ddl = True
