@@ -1,47 +1,47 @@
-from __future__ import with_statement
-
-from mako.template import Template
 import sys
 import os
 import textwrap
-from sqlalchemy.engine import url
-from sqlalchemy import util as sqla_util
-import imp
 import warnings
 import re
 import inspect
-import time
-import random
 import uuid
 
+from mako.template import Template
+from sqlalchemy.engine import url
+from sqlalchemy import __version__
+
+from .compat import callable, exec_, load_module, binary_type
+
 class CommandError(Exception):
     pass
 
-from sqlalchemy import __version__
 def _safe_int(value):
     try:
         return int(value)
     except:
-        return 0
-_vers = tuple([_safe_int(x) for x in __version__.split(".")])
-sqla_06 = _vers > (0, 6)
-sqla_07 = _vers > (0, 7)
-if not sqla_06:
+        return value
+_vers = tuple([_safe_int(x) for x in re.findall(r'(\d+|[abc]\d)', __version__)])
+sqla_07 = _vers > (0, 7, 2)
+sqla_08 = _vers >= (0, 8, 0, 'b2')
+sqla_09 = _vers >= (0, 9, 0)
+if not sqla_07:
     raise CommandError(
-            "SQLAlchemy 0.6 or greater is required. "
-            "Version 0.7 or above required for full featureset.")
+            "SQLAlchemy 0.7.3 or greater is required. ")
+
+from sqlalchemy.util import format_argspec_plus, update_wrapper
+from sqlalchemy.util.compat import inspect_getfullargspec
+
 
-def requires_07(feature):
-    if not sqla_07:
-        raise CommandError(
-            "The %s feature requires "
-            "SQLAlchemy 0.7 or greater."
-            % feature
-        )
 try:
-    width = int(os.environ['COLUMNS'])
-except (KeyError, ValueError):
-    width = 80
+    import fcntl
+    import termios
+    import struct
+    ioctl = fcntl.ioctl(0, termios.TIOCGWINSZ,
+                           struct.pack('HHHH', 0, 0, 0, 0))
+    _h, TERMWIDTH, _hp, _wp = struct.unpack('HHHH', ioctl)
+except (ImportError, IOError):
+    TERMWIDTH = None
+
 
 def template_to_file(template_file, dest, **kw):
     with open(dest, 'w') as f:
@@ -86,7 +86,7 @@ def create_module_class_proxy(cls, globals_, locals_):
             num_defaults += len(spec[3])
         name_args = spec[0]
         if num_defaults:
-            defaulted_vals = name_args[0-num_defaults:]
+            defaulted_vals = name_args[0 - num_defaults:]
         else:
             defaulted_vals = ()
 
@@ -115,13 +115,13 @@ def create_module_class_proxy(cls, globals_, locals_):
             return _proxy.%(name)s(%(apply_kw)s)
             e
         """ % {
-            'name':name,
-            'args':args[1:-1],
-            'apply_kw':apply_kw[1:-1],
-            'doc':fn.__doc__,
+            'name': name,
+            'args': args[1:-1],
+            'apply_kw': apply_kw[1:-1],
+            'doc': fn.__doc__,
         })
         lcl = {}
-        exec func_text in globals_, lcl
+        exec_(func_text, globals_, lcl)
         return lcl[name]
 
     for methname in dir(cls):
@@ -131,6 +131,14 @@ def create_module_class_proxy(cls, globals_, locals_):
             else:
                 attr_names.add(methname)
 
+def write_outstream(stream, *text):
+    encoding = getattr(stream, 'encoding', 'ascii') or 'ascii'
+    for t in text:
+        if not isinstance(t, binary_type):
+            t = t.encode(encoding, 'replace')
+        t = t.decode(encoding)
+        stream.write(t)
+
 def coerce_resource_to_filename(fname):
     """Interpret a filename as either a filesystem location or as a package resource.
 
@@ -144,13 +152,13 @@ def coerce_resource_to_filename(fname):
     return fname
 
 def status(_statmsg, fn, *arg, **kw):
-    msg(_statmsg + "...", False)
+    msg(_statmsg + " ...", False)
     try:
         ret = fn(*arg, **kw)
-        sys.stdout.write("done\n")
+        write_outstream(sys.stdout, " done\n")
         return ret
     except:
-        sys.stdout.write("FAILED\n")
+        write_outstream(sys.stdout, " FAILED\n")
         raise
 
 def err(message):
@@ -171,18 +179,24 @@ def warn(msg):
     warnings.warn(msg)
 
 def msg(msg, newline=True):
-    lines = textwrap.wrap(msg, width)
-    if len(lines) > 1:
-        for line in lines[0:-1]:
-            sys.stdout.write("  " +line + "\n")
-    sys.stdout.write("  " + lines[-1] + ("\n" if newline else ""))
+    if TERMWIDTH is None:
+        write_outstream(sys.stdout, msg)
+        if newline:
+            write_outstream(sys.stdout, "\n")
+    else:
+        # left indent output lines
+        lines = textwrap.wrap(msg, TERMWIDTH)
+        if len(lines) > 1:
+            for line in lines[0:-1]:
+                write_outstream(sys.stdout, "  ", line, "\n")
+        write_outstream(sys.stdout, "  ", lines[-1], ("\n" if newline else ""))
 
 def load_python_file(dir_, filename):
     """Load a file from the given path as a Python module."""
 
     module_id = re.sub(r'\W', "_", filename)
     path = os.path.join(dir_, filename)
-    module = imp.load_source(module_id, path, open(path, 'rb'))
+    module = load_module(module_id, path)
     del sys.modules[module_id]
     return module
 
@@ -250,3 +264,42 @@ class immutabledict(dict):
 
     def __repr__(self):
         return "immutabledict(%s)" % dict.__repr__(self)
+
+
+def _with_legacy_names(translations):
+    def decorate(fn):
+
+        spec = inspect_getfullargspec(fn)
+        metadata = dict(target='target', fn='fn')
+        metadata.update(format_argspec_plus(spec, grouped=False))
+
+        has_keywords = bool(spec[2])
+
+        if not has_keywords:
+            metadata['args'] += ", **kw"
+            metadata['apply_kw'] += ", **kw"
+
+        def go(*arg, **kw):
+            names = set(kw).difference(spec[0])
+            for oldname, newname in translations:
+                if oldname in kw:
+                    kw[newname] = kw.pop(oldname)
+                    names.discard(oldname)
+
+                    warnings.warn(
+                        "Argument '%s' is now named '%s' for function '%s'" %
+                        (oldname, newname, fn.__name__))
+            if not has_keywords and names:
+                raise TypeError("Unknown arguments: %s" % ", ".join(names))
+            return fn(*arg, **kw)
+
+        code = 'lambda %(args)s: %(target)s(%(apply_kw)s)' % (
+                metadata)
+        decorated = eval(code, {"target": go})
+        decorated.__defaults__ = getattr(fn, '__func__', fn).__defaults__
+        return update_wrapper(decorated, fn)
+
+    return decorate
+
+
+
