@@ -1,11 +1,12 @@
-from tests import op_fixture, _sqlite_testing_config, eq_, assert_raises_message
-from alembic import op
-from sqlalchemy import Integer, \
-            UniqueConstraint, String
-from sqlalchemy.sql import table, column
 from unittest import TestCase
+
+from alembic import op
+from sqlalchemy import Integer, String
+from sqlalchemy.sql import table, column
 from sqlalchemy import Table, Column, MetaData
 
+from . import op_fixture, eq_, assert_raises_message
+
 def _table_fixture(dialect, as_sql):
     context = op_fixture(dialect, as_sql)
     t1 = table("ins_table",
